`ifndef AXI4_MM_DRIVER_MASTER_SVH
`define AXI4_MM_DRIVER_MASTER_SVH

/* Class: axi4_mm_driver_master
 *
 * AXI4-MM Master interface driver.
 *
 * Parameters:
 *  DATA_BYTES      - Number of bytes for data signals.
 *  ID_WIDTH        - Number of bits for id signals.
 *  ADDRESS_WIDTH   - Number of bits for address signals.
 */
class axi4_mm_driver_master #(
    int DATA_BYTES      = 4,
    int ID_WIDTH        = 8,
    int ADDRESS_WIDTH   = 1
);
    typedef virtual axi4_mm_if #(
        .DATA_BYTES     (DATA_BYTES),
        .ADDRESS_WIDTH  (ADDRESS_WIDTH),
        .ID_WIDTH       (ID_WIDTH)
    ) .cb_master_modport vif_t;

    /* Parametrized data types from axi4_mm package */
    typedef axi4_mm_pkg::request_t    # (.ID_WIDTH(ID_WIDTH), .ADDRESS_WIDTH(ADDRESS_WIDTH), .BYTES(DATA_BYTES) )     request_t;
    typedef axi4_mm_pkg::wdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                    wdata_t;
    typedef axi4_mm_pkg::bresp_t      # (.ID_WIDTH(ID_WIDTH) )                                                        bresp_t;
    typedef axi4_mm_pkg::rdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                    rdata_t;
    typedef axi4_mm_pkg::size_t                                                                                       size_t;
    typedef axi4_mm_pkg::burst_t                                                                                      burst_t; 
    typedef axi4_mm_pkg::response_t                                                                                   response_t;
    typedef bresp_t                                                                                                         bresp_array_t[];
    typedef rdata_t                                                                                                         rdata_array_t[];

    /* Internal logic AXI4-MM output signals (from driver to master interface) */
    local logic [     ID_WIDTH-1:0]         bid;
    local logic [              1:0]         bresp;
    local logic [     ID_WIDTH-1:0]         rid;
    local logic [   DATA_BYTES-1:0][7:0]    rdata;
    local logic [              1:0]         rresp;
    local logic                             rlast;

    /* Interface */
    local vif_t                             m_vif;
    local int                               m_timeout;

    /* Function: new
     * Constructor. Assigns the AXI4-MM master interface to object.
     */
    extern function         new(vif_t vif);

    /* Function: set_timeout 
     * Sets the amount of clock cycles that the driver will wait before unholding the bus if there is no valid or ready asserted.
     * Input: amount of clock cycles to wait (-1 : wait forever)
     */
    extern function void    set_timeout(int value);

    /* Task: aclk_posedge
     * Wait for rising edge of clock. (Clock period delay).
     * Input: count - number of clock cycles to wait.
     */    
    extern task             aclk_posedge(int count = 1);

    /* Task: reset
     * Initializes AXI4-MM master interface that is connected to the object into reset state.
     */
    extern task             reset();
    
    /* Task: write_address
     * Receives AXI4-MM write address transaction (AW channel).
     * Input: reference to request_t object which will be used to store AW transaction data.
     */
    extern task             write_address           (ref request_t request);

    /* Task: write_data
     * Receives AXI4-MM write transaction (W channel).
     * Input: reference to wdata_t object which will be used to store W transaction data.
     */
    extern task             write_data              (ref wdata_t data); 

    /* Task: write_response
     * Transmit AXI4-MM write response transaction (B channel).
     * Input: bresp_t object which contains B channel transaction data.
     */
    extern task             write_response          (bresp_t response); 

    /* Task: write
     * AXI4-MM write transaction. Sequentially receives AW and W data, then sends B data.
     * Input:   request     - reference to request_t object which will be used to store AW transaction data.
     *          data        - reference to wdata_t object which will be used to store W transaction data.
     *          response    - bresp_t object which contains B channel transaction data.
     */
    extern task             write                   (ref request_t request, ref wdata_t data, bresp_t response);

    /* Task: write_multiple_address
     * Receive AXI4-MM write address burst transaction.
     * Input: reference to queue of request_t objects whose will be used to store AW transaction data.
     */
    extern task             write_multiple_address  (ref request_t request[]);

    /* Task: write_multiple_data
     * Receive AXI4-MM write data burst transaction.
     * Input: reference to queue of wdata_t objects whose will be used to store W transaction data.
     */
    extern task             write_multiple_data     (ref wdata_t data[]);    
    
    /* Task: write_multiple_response
     * Transmit AXI4-MM write response burst transaction (B channel).
     * Input: queue of bresp_t objects whose contain B channel transaction data.
     */    
    extern task             write_multiple_response (const ref bresp_array_t response);

    /* Task: write_multiple
     * AXI4-MM write burst transaction. Sequentially receives AW and W data, then sends B data.
     * Input:   request     - reference to queue of request_t objects whose will be used to store AW transaction data.
     *          data        - reference to queue of wdata_t objects whose will be used to store W transaction data.
     *          response    - queue of bresp_t objects whose contain B channel transaction data.
     */
    extern task             write_multiple          (ref request_t request[], ref wdata_t data[], const ref bresp_array_t response);

    /* Task: write_ready
     * Sets write idle state (AWREADY and WREADY)
     * Input: Idle on (value = 0) or off (value = 1)
     */
    extern task             write_ready             (bit value = 1);

    /* Task: read_address
     * Receives AXI4-MM read address transaction (AR channel).
     * Input: reference to request_t object which will be used to store AR transaction data.
     */ 
    extern task             read_address            (ref request_t request);

    /* Task: read_data
     * Transmits AXI4-MM read transaction (R channel).
     * Input: rdata_t object which contains R transaction data.
     */    
    extern task             read_data               (rdata_t data);

    /* Task: read
     * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
     * Input:   request     - reference to request_t object which will be used to store AR transaction data.
     *          data        - rdata_t object which contains R transaction data.
     */
    extern task             read                    (ref request_t request, rdata_t data);

    /* Task: read_multiple_address
     * Receive AXI4-MM read address burst transaction.
     * Input: reference to queue of request_t objects whose will be used to store AR transaction data.
     */
    extern task             read_multiple_address   (ref request_t request[]);

    /* Task: read_multiple_data
     * Transmit AXI4-MM read data burst transaction.
     * Input: queue of rdata_t objects whose contain R transaction data.
     */
    extern task             read_multiple_data      (const ref rdata_array_t data);

    /* Task: read_multiple
     * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
     * Input:   request     - reference to queue of request_t objects whose will be used to store AR transaction data.
     *          data        - queue of rdata_t objects whose contain R transaction data.
     */
    extern task             read_multiple           (ref request_t request[], const ref rdata_array_t data);

    /* Task: read_ready
     * Sets read idle state (ARREADY and RREADY)
     * Input: Idle on (value = 0) or off (value = 1)
     */
    extern task             read_ready              (bit value = 1);

    /* Task: assign_signals
     * Assign local signals to interface signals (must be called when the signals are changed i.e. by assign_channel() functions)
     */
    task assign_signals;
        m_vif.cb_master.bid     <= bid;
        m_vif.cb_master.buser   <= '0;
        m_vif.cb_master.bresp   <= bresp;
        m_vif.cb_master.rid     <= rid;
        m_vif.cb_master.rdata   <= rdata;
        m_vif.cb_master.rresp   <= rresp;
        m_vif.cb_master.rlast   <= rlast;
    endtask    
endclass

/* Function: new
 * Constructor. Assigns the AXI4-MM master interface to object.
 */
function axi4_mm_driver_master::new(vif_t vif);
    m_vif       = vif;
    m_timeout   = 0;
endfunction

/* Function: set_timeout 
 * Sets the amount of clock cycles that the driver will wait before unholding the bus if there is no valid or ready asserted.
 * Input: amount of clock cycles to wait (-1 : wait forever)
 */
function void axi4_mm_driver_master::set_timeout(int value);
    m_timeout   = value;
endfunction

/* Task: reset
 * Initializes AXI4-MM master interface that is connected to the object into reset state.
 */
task axi4_mm_driver_master::reset();
    m_vif.cb_master.awready <= 1'b0;    
    m_vif.cb_master.wready  <= 1'b0;  
    m_vif.cb_master.bid     <= 'X;
    m_vif.cb_master.bresp   <= response_t'('X);
    m_vif.cb_master.buser   <= 'X;
    m_vif.cb_master.bvalid  <= 1'b0; 
    m_vif.cb_master.arready <= 1'b0;
    m_vif.cb_master.rid     <= 'X;
    m_vif.cb_master.rdata   <= 'X;
    m_vif.cb_master.rresp   <= response_t'('X);
    m_vif.cb_master.rlast   <= 'X;
    m_vif.cb_master.ruser   <= 'X;
    m_vif.cb_master.rvalid  <= 1'b0;
endtask

/* Task: write_address
 * Receives AXI4-MM write address transaction (AW channel).
 * Input: reference to request_t object which will be used to store AW transaction data.
 */
task axi4_mm_driver_master::write_address(ref request_t request);
    request_t   _request;
    bit         is_running      = 1;
    int         timeout         = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.awready     <= 1;
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.awvalid) && (1'b1 === m_vif.cb_master.awready)) begin
            _request    = new(m_vif.cb_master.awid, m_vif.cb_master.awaddr, m_vif.cb_master.awlen, size_t'(m_vif.cb_master.awsize), burst_t'(m_vif.cb_master.awburst));
            is_running  = 0;
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;  
    end
    m_vif.cb_master.awready <= 0;
    request                 = _request;
endtask

/* Task: write_data
 * Receives AXI4-MM write transaction (W channel).
 * Input: reference to wdata_t object which will be used to store W transaction data.
 */
task axi4_mm_driver_master::write_data(ref wdata_t data);
    wdata_t _data           = new();  
    bit is_running          = 1;
    bit data_avaiable       = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.wready  <= 1;
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if (!data_avaiable) is_running = 0;
        if ((1'b1 === m_vif.cb_master.wvalid) && (1'b1 === m_vif.cb_master.wready)) begin
            _data.set_id(m_vif.cb_master.wid);
            _data.write (m_vif.cb_master.wdata, m_vif.cb_master.wstrb);
            if (1'b1 === m_vif.cb_master.wlast) data_avaiable = 0;
        end
        else if ( timeout > 0)  --timeout;
        else if (timeout == 0)  is_running = 0;
    end
    m_vif.cb_master.wready  <= 0;
    data                    = _data;
endtask

/* Task: write_response
 * Transmit AXI4-MM write response transaction (B channel).
 * Input: bresp_t object which contains B channel transaction data.
 */
task axi4_mm_driver_master::write_response(bresp_t response);
    bit is_running          = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.bvalid  <= 1;
    response.assign_channel(bid, bresp);
    assign_signals();
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.bvalid) && (1'b1 === m_vif.cb_master.bready)) is_running = 0;
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;
    end
    m_vif.cb_master.bvalid <= 0;
endtask

/* Task: write
 * AXI4-MM write transaction. Sequentially receives AW and W data, then sends B data.
 * Input:   request     - reference to request_t object which will be used to store AW transaction data.
 *          data        - reference to wdata_t object which will be used to store W transaction data.
 *          response    - bresp_t object which contains B channel transaction data.
 */
task axi4_mm_driver_master::write (ref request_t request, ref wdata_t data, bresp_t response);
    write_address   (request);
    write_data      (data);
    write_response  (response);
endtask

/* Task: write_multiple_address
 * Receive AXI4-MM write address burst transaction.
 * Input: reference to queue of request_t objects whose will be used to store AW transaction data.
 */
task axi4_mm_driver_master::write_multiple_address (ref request_t request[]);
    request_t _request[$];
    request_t current_request;
    bit is_running              = 0;
    int timeout                 = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.awready     <= 1;
    while (1'b1 === m_vif.areset_n) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.awvalid) && (1'b1 === m_vif.cb_master.awready)) begin
            current_request = new(m_vif.cb_master.awid, m_vif.cb_master.awaddr, m_vif.cb_master.awlen, size_t'(m_vif.cb_master.awsize), burst_t'(m_vif.cb_master.awburst));
            _request.push_back(current_request);
            is_running      = 1;
        end 
        if((1'b0 === m_vif.cb_master.awvalid) && is_running == 1)   break;
        else if ( timeout > 0)                                      --timeout;
        else if (timeout == 0)                                      is_running = 0; 
    end
    m_vif.cb_master.awready <= 0;
    request                 = _request;
endtask

/* Task: write_multiple_data
 * Receive AXI4-MM write data burst transaction.
 * Input: reference to queue of wdata_t objects whose will be used to store W transaction data.
 */
task axi4_mm_driver_master::write_multiple_data (ref wdata_t data[]);   
    wdata_t _data[$];
    wdata_t current_data;  
    bit     is_running      = 0;
    bit     data_avaiable   = 0;
    int     timeout         = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.wready  <= 1;
    while (1'b1 === m_vif.areset_n) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.wvalid) && (1'b1 === m_vif.cb_master.wready)) begin
            is_running = 1;
            if (data_avaiable == 0) begin
                current_data    = new   (m_vif.cb_master.wid);
                current_data.write      (m_vif.cb_master.wdata, m_vif.cb_master.wstrb);
                data_avaiable   = 1;
            end
            else begin 
                current_data.write      (m_vif.cb_master.wdata, m_vif.cb_master.wstrb);
            end
            if (1'b1 === m_vif.cb_master.wlast) begin
                _data.push_back         (current_data);
                data_avaiable   = 0;
            end 
        end
        else if ( timeout > 0)  --timeout;
        else if (timeout == 0)  break;
        if (is_running == 1 && 1'b0 === m_vif.cb_master.wvalid) break;
    end
    m_vif.cb_master.wready  <= 0;
    data                    = _data;    
endtask

/* Task: write_multiple_response
 * Transmit AXI4-MM write response burst transaction (B channel).
 * Input: queue of bresp_t objects whose contain B channel transaction data.
 */
task axi4_mm_driver_master::write_multiple_response (const ref bresp_array_t response);
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    int index               = 0;
    m_vif.cb_master.bvalid  <= 1;
    response[0].assign_channel(bid, bresp);
    assign_signals();
    while (++index != response.size() && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.bvalid) && (1'b1 === m_vif.cb_master.bready)) begin
            response[index].assign_channel(bid, bresp);
            assign_signals();
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) break;
    end
    aclk_posedge();
    m_vif.cb_master.bvalid <= 0;  
endtask

/* Task: write_multiple
 * AXI4-MM write burst transaction. Sequentially receives AW and W data, then sends B data.
 * Input:   request     - reference to queue of request_t objects whose will be used to store AW transaction data.
 *          data        - reference to queue of wdata_t objects whose will be used to store W transaction data.
 *          response    - queue of bresp_t objects whose contain B channel transaction data.
 */
task axi4_mm_driver_master::write_multiple (ref request_t request[], ref wdata_t data[], const ref bresp_array_t response);    
    write_multiple_address  (request);
    write_multiple_data     (data);
    write_multiple_response (response);
endtask

/* Task: write_ready
 * Sets write idle state (AWREADY and WREADY)
 * Input: Idle on (value = 0) or off (value = 1)
 */
task axi4_mm_driver_master::write_ready (bit value = 1);
    m_vif.cb_master.wready  <= value;
    m_vif.cb_master.awready <= value;
endtask

/* Task: read_address
 * Receives AXI4-MM read address transaction (AR channel).
 * Input: reference to request_t object which will be used to store AR transaction data.
 */ 
task axi4_mm_driver_master::read_address (ref request_t request);
    request_t   _request;
    bit         is_running  = 1;
    int         timeout     = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.arready <= 1;
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.arvalid) && (1'b1 === m_vif.cb_master.arready)) begin
            _request    = new(m_vif.cb_master.arid, m_vif.cb_master.araddr, m_vif.cb_master.arlen, size_t'(m_vif.cb_master.arsize), burst_t'(m_vif.cb_master.arburst));
            is_running  = 0;
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;
    end
    m_vif.cb_master.arready <= 0;
    request                 = _request;
endtask

/* Task: read_data
 * Transmits AXI4-MM read transaction (R channel).
 * Input: rdata_t object which contains R transaction data.
 */
task axi4_mm_driver_master::read_data (rdata_t data);
    bit is_running          = 1;
    bit data_avaiable       = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    data.assign_channel(rid, rdata, rlast, rresp);
    assign_signals();
    m_vif.cb_master.rvalid  <= 1;
    if (!data.get_last()) begin
        while (is_running && (1'b1 === m_vif.areset_n)) begin
            aclk_posedge();
            if (data_avaiable == 0) is_running = 0;
            else begin
                if ((1'b1 === m_vif.cb_master.rvalid) && (1'b1 === m_vif.cb_master.rready)) begin
                    data.assign_channel(rid, rdata, rlast, rresp);
                    assign_signals();
                    if(data.get_last()) data_avaiable = 0;
                end
                else if (timeout > 0)   --timeout;
                else if (timeout == 0)  is_running = 0;
            end
        end
    end
    else begin
        aclk_posedge();
        is_running      = 0;
        data_avaiable   = 0;
    end
    m_vif.cb_master.rvalid <= 0;
endtask

/* Task: read
 * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
 * Input:   request     - reference to request_t object which will be used to store AR transaction data.
 *          data        - rdata_t object which contains R transaction data.
 */
task axi4_mm_driver_master::read (ref request_t request, rdata_t data);
    read_address(request);
    read_data   (data);
endtask

/* Task: read_multiple_address
 * Receive AXI4-MM read address burst transaction.
 * Input: reference to queue of request_t objects whose will be used to store AR transaction data.
 */
task axi4_mm_driver_master::read_multiple_address (ref request_t request[]);
    request_t   _request[$];
    request_t   current_request;
    bit         is_running      = 0;
    int         timeout         = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_master.arready     <= 1;
    while (1'b1 === m_vif.areset_n) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_master.arvalid) && (1'b1 === m_vif.cb_master.arready)) begin
            current_request = new(m_vif.cb_master.arid, m_vif.cb_master.araddr, m_vif.cb_master.arlen, size_t'(m_vif.cb_master.arsize), burst_t'(m_vif.cb_master.arburst));
            _request.push_back(current_request);
            is_running      = 1;
        end 
        if((1'b0 === m_vif.cb_master.arvalid) && is_running == 1) break;
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0; 
    end
    m_vif.cb_master.arready <= 0;
    request                 = _request;
endtask

/* Task: read_multiple_data
 * Transmit AXI4-MM read data burst transaction.
 * Input: queue of rdata_t objects whose contain R transaction data.
 */
task axi4_mm_driver_master::read_multiple_data(const ref rdata_array_t data);
    int timeout = (m_timeout > 0) ? m_timeout : -1;
    int index   = 0;
    while(index < data.size() && (1'b1 === m_vif.areset_n)) begin
        data[index].assign_channel(rid, rdata, rlast, rresp);
        assign_signals();
        m_vif.cb_master.rvalid   <= 1;
        while (1'b1 === m_vif.areset_n) begin
            aclk_posedge();
            if ((1'b1 === m_vif.cb_master.rvalid) && (1'b1 === m_vif.cb_master.rready)) begin
                data[index].assign_channel(rid, rdata, rlast, rresp);
                assign_signals();
                if(data[index].get_last()) break;
            end
            else if ( timeout > 0) --timeout;
            else if (timeout == 0) break;
        end
        aclk_posedge();
        index++;
    end
    m_vif.cb_master.rvalid <= 0;
endtask

/* Task: read_multiple
 * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
 * Input:   request     - reference to queue of request_t objects whose will be used to store AR transaction data.
 *          data        - queue of rdata_t objects whose contain R transaction data.
 */
task axi4_mm_driver_master::read_multiple(ref request_t request[], const ref rdata_array_t data);
    read_multiple_address   (request);
    read_multiple_data      (data);
endtask

/* Task: read_ready
 * Sets read idle state (ARREADY and RREADY)
 * Input: Idle on (value = 0) or off (value = 1)
 */
task axi4_mm_driver_master::read_ready(bit value = 1);
    m_vif.cb_master.arready <= value;
endtask

/* Task: aclk_posedge
 * Wait for rising edge of clock. (Clock period delay).
 * Input: count - number of clock cycles to wait.
 */
task axi4_mm_driver_master::aclk_posedge(int count = 1);
    while (count--) begin
        @(m_vif.cb_master);
    end
endtask

`endif /* AXI4_MM_DRIVER_MASTER_SVH */
