`ifndef AXI4_MM_DRIVER_SLAVE_SVH
`define AXI4_MM_DRIVER_SLAVE_SVH

/* Class: axi4_mm_driver_slave
 *
 * AXI4-MM Slave interface driver.
 *
 * Parameters:
 *  DATA_BYTES      - Number of bytes for data signals.
 *  ID_WIDTH        - Number of bits for id signals.
 *  ADDRESS_WIDTH   - Number of bits for address signals.
 */
class axi4_mm_driver_slave #(
    int DATA_BYTES      = 4,
    int ID_WIDTH        = 8,
    int ADDRESS_WIDTH   = 1
);
    /* Parametrized AXI4-MM interface type */
    typedef virtual axi4_mm_if #(
        .DATA_BYTES     (DATA_BYTES),
        .ADDRESS_WIDTH  (ADDRESS_WIDTH),
        .ID_WIDTH       (ID_WIDTH)
    ) .cb_slave_modport vif_t;

    /* Parametrized data types from axi4_mm package */
    typedef axi4_mm_pkg::request_t    # (.ID_WIDTH(ID_WIDTH), .ADDRESS_WIDTH(ADDRESS_WIDTH), .BYTES(DATA_BYTES) )     request_t;
    typedef request_t                                                                                                       request_array_t[];
    typedef axi4_mm_pkg::wdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                    wdata_t;
    typedef wdata_t                                                                                                         wdata_array_t[];
    typedef axi4_mm_pkg::bresp_t      # (.ID_WIDTH(ID_WIDTH) )                                                        bresp_t;
    typedef axi4_mm_pkg::rdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                    rdata_t;
    typedef axi4_mm_pkg::response_t                                                                                   response_t;

    /* Internal logic AXI4-MM output signals (from driver to slave interface) */
    local logic [     ID_WIDTH-1:0]         awid;
    local logic [ADDRESS_WIDTH-1:0]         awaddr;
    local logic [              7:0]         awlen;
    local logic [              2:0]         awsize;
    local logic [              1:0]         awburst;
    local logic [     ID_WIDTH-1:0]         wid;
    local logic [   DATA_BYTES-1:0][7:0]    wdata;
    local logic [   DATA_BYTES-1:0]         wstrb;
    local logic                             wlast;
    local logic [     ID_WIDTH-1:0]         arid;
    local logic [ADDRESS_WIDTH-1:0]         araddr;
    local logic [              7:0]         arlen;
    local logic [              2:0]         arsize;
    local logic [              1:0]         arburst;

    /* Interface */
    local vif_t                             m_vif;
    local int                               m_timeout;

    /* Function: new
     * Constructor. Assigns the AXI4-MM slave interface to object.
     */
    extern function         new (vif_t vif);
    
    /* Function: set_timeout 
     * Sets the amount of clock cycles that the driver will wait before unholding the bus if there is no valid or ready asserted.
     * Input: amount of clock cycles to wait (-1 : wait forever)
     */
    extern function void    set_timeout(int value);

    /* Task: reset
     * Initializes AXI4-MM slave interface that is connected to the object into reset state.
     */
    extern task             reset();

    /* Task: write_address
     * Transmits AXI4-MM write address transaction (AW channel).
     * Input: request_t object which contains AW transaction data.
     */
    extern task             write_address           (request_t request);

    /* Task: write_data
     * Transmit AXI4-MM write transaction (W channel).
     * Input: wdata_t object which contains W transaction data.
     */
    extern task             write_data              (wdata_t data);   

    /* Task: write_response
     * Receives AXI4-MM write response transaction (B channel).
     * Input: reference to bresp_t object which will be used to store B transaction data.
     */
    extern task             write_response          (ref bresp_t response);

    /* Task: write
     * AXI4-MM write transaction. Sequentially transmits AW and W data, then receives B data.
     * Input:   request     - request_t object which contains AW channel transaction data.
     *          data        - wdata_t object which contains W channel transaction data.
     *          response    - bresp_t object which will be used to store B transaction data.
     */
    extern task             write                   (request_t request, wdata_t data, ref bresp_t response);

    /* Task: write_multiple_address
     * Transmit AXI4-MM write address burst transaction.
     * Input: queue of wdata_t objects whose contain AW channel transaction data.
     */
    extern task             write_multiple_address  (const ref request_array_t request);

    /* Task: write_multiple_data
     * Transmit AXI4-MM write data burst transaction.
     * Input: queue of wdata_t objects whose contain W channel transaction data.
     */
    extern task             write_multiple_data     (const ref wdata_array_t data);   

    /* Task: write_multiple_response
     * Receive AXI4-MM write response burst transaction (B channel).
     * Input: reference to queue of bresp_t objects whose will be used to store B transaction data.
     */
    extern task             write_multiple_response (ref bresp_t response[]);    

    /* Task: write_multiple
     * AXI4-MM write burst transaction. Sequentially receives AW and W data, then sends B data.
     * Input:   request     - queue of wdata_t objects whose contain AW channel transaction data.
     *          data        - queue of wdata_t objects whose contain W channel transaction data.
     *          response    - reference to queue of bresp_t objects whose will be used to store B transaction data.
     */
    extern task             write_multiple          (const ref request_array_t request, const ref wdata_array_t data, ref bresp_t response[]);          

    /* Task: write_ready
     * Sets write idle state (BREADY)
     * Input: Idle on (value = 0) or off (value = 1)
     */
    extern task             write_ready             (bit value = 1);

    /* Task: read_address
     * Transmits AXI4-MM read address transaction (AR channel).
     * Input: request_t object which contains AR channel transaction data.
     */
    extern task             read_address            (request_t request);

    /* Task: read_data
     * Transmits AXI4-MM read transaction (R channel).
     * Input: reference to rdata_t object which will be used to store R transaction data.
     */
    extern task             read_data               (ref rdata_t data);
 
    /* Task: read
     * AXI4-MM read transaction. Sequentially transmits AR data, then receives R data.
     * Input:   request     - request_t object which contains AR transaction data..
     *          data        - reference to rdata_t object which will be used to store R transaction data.
     */
    extern task             read                    (request_t request, ref rdata_t data);

    /* Task: read_multiple_address
     * Transmits AXI4-MM read address burst transaction.
     * Input: queue of request_t objects whose contain AR transaction data.
     */
    extern task             read_multiple_address   (const ref request_array_t request);

    /* Task: read_multiple_data
     * Transmit AXI4-MM read data burst transaction.
     * Input: reference to queue of rdata_t objects whose will be used to store R transaction data.
     */
    extern task             read_multiple_data      (ref rdata_t data[]);    

    /* Task: read_multiple
     * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
     * Input:   request     - queue of request_t objects whose contain AR transaction data.
     *          data        - reference to queue of rdata_t objects whose will be used to store R transaction data.
     */
    extern task             read_multiple           (const ref request_array_t request, ref rdata_t data[]);

    /* Task: read_ready
     * Sets read idle state (RREADY)
     * Input: Idle on (value = 0) or off (value = 1)
     */
    extern task             read_ready              (bit value = 1);

    /* Task: aclk_posedge
     * Wait for rising edge of clock. (Clock period delay).
     * Input: count - number of clock cycles to wait.
     */
    extern task             aclk_posedge            (int count = 1);

    /* Task: assign_signals
     * Assign local signals to interface signals (must be called when the signals are changed i.e. by assign_channel() functions)
     */
    task assign_signals;
        m_vif.cb_slave.awid     <= awid;
        m_vif.cb_slave.awaddr   <= awaddr;
        m_vif.cb_slave.awlen    <= awlen;
        m_vif.cb_slave.awsize   <= awsize;
        m_vif.cb_slave.awburst  <= awburst;
        m_vif.cb_slave.awlock   <= axi4_mm_pkg::LOCK_NORMAL_ACCESS;      
        m_vif.cb_slave.awcache  <= axi4_mm_pkg::DEFAULT_CACHE;    
        m_vif.cb_slave.awprot   <= axi4_mm_pkg::DEFAULT_DATA_ACCESS;     
        m_vif.cb_slave.awqos    <= '0;     
        m_vif.cb_slave.awregion <= '0;    
        m_vif.cb_slave.awuser   <= '0;
        m_vif.cb_slave.wid      <= wid;
        m_vif.cb_slave.wdata    <= wdata;
        m_vif.cb_slave.wstrb    <= wstrb;
        m_vif.cb_slave.wlast    <= wlast;
        m_vif.cb_slave.wuser    <= '0;
        m_vif.cb_slave.arid     <= arid;     
        m_vif.cb_slave.araddr   <= araddr;    
        m_vif.cb_slave.arlen    <= arlen;      
        m_vif.cb_slave.arsize   <= arsize;     
        m_vif.cb_slave.arburst  <= arburst;     
        m_vif.cb_slave.arlock   <= axi4_mm_pkg::LOCK_NORMAL_ACCESS;      
        m_vif.cb_slave.arcache  <= axi4_mm_pkg::DEFAULT_CACHE;    
        m_vif.cb_slave.arprot   <= axi4_mm_pkg::DEFAULT_DATA_ACCESS;     
        m_vif.cb_slave.arqos    <= '0;     
        m_vif.cb_slave.arregion <= '0;    
        m_vif.cb_slave.aruser   <= '0;
    endtask
endclass

/* Function: new
 * Constructor. Assigns the AXI4-MM slave interface to object.
 */
function axi4_mm_driver_slave::new(vif_t vif);
    m_vif       = vif;
    m_timeout   = 0;
endfunction

/* Function: set_timeout 
 * Sets the amount of clock cycles that the driver will wait before unholding the bus if there is no valid or ready asserted.
 * Input: amount of clock cycles to wait (-1 : wait forever)
 */
function void axi4_mm_driver_slave::set_timeout(int value);
    m_timeout   = value;
endfunction

/* Task: reset
 * Initializes AXI4-MM slave interface that is connected to the object into reset state.
 */
task axi4_mm_driver_slave::reset();
    m_vif.cb_slave.wvalid   <= 1'b0;
    m_vif.cb_slave.wid      <= 'X;
    m_vif.cb_slave.wdata    <= 'X;
    m_vif.cb_slave.wstrb    <= 'X;
    m_vif.cb_slave.wlast    <= 'X;
    m_vif.cb_slave.wuser    <= 'X;
    m_vif.cb_slave.awvalid  <= 1'b0; 
    m_vif.cb_slave.awid     <= 'X;     
    m_vif.cb_slave.awaddr   <= 'X;    
    m_vif.cb_slave.awlen    <= 'X;      
    m_vif.cb_slave.awsize   <= 'X;     
    m_vif.cb_slave.awburst  <= 'X;     
    m_vif.cb_slave.awlock   <= 'X;      
    m_vif.cb_slave.awcache  <= 'X;    
    m_vif.cb_slave.awprot   <= 'X;     
    m_vif.cb_slave.awqos    <= 'X;     
    m_vif.cb_slave.awregion <= 'X;    
    m_vif.cb_slave.awuser   <= 'X;
    m_vif.cb_slave.bready   <= 1'b0;         
    m_vif.cb_slave.arvalid  <= 1'b0;  
    m_vif.cb_slave.arid     <= 'X;   
    m_vif.cb_slave.araddr   <= 'X;   
    m_vif.cb_slave.arlen    <= 'X;   
    m_vif.cb_slave.arsize   <= 'X;  
    m_vif.cb_slave.arburst  <= 'X;   
    m_vif.cb_slave.arlock   <= 'X;  
    m_vif.cb_slave.arcache  <= 'X;    
    m_vif.cb_slave.arprot   <= 'X;     
    m_vif.cb_slave.arqos    <= 'X;      
    m_vif.cb_slave.arregion <= 'X;    
    m_vif.cb_slave.aruser   <= 'X;            
    m_vif.cb_slave.rready   <= 1'b0;
endtask

/* Task: write_address
 * Transmits AXI4-MM write address transaction (AW channel).
 * Input: request_t object which contains AW transaction data.
 */
task axi4_mm_driver_slave::write_address(request_t request);
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_slave.awvalid  <= 1'b1; 
    request.assign_channel(awid, awaddr, awlen, awsize, awburst);    
    assign_signals();
    while ((1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.awvalid) && (1'b1 === m_vif.cb_slave.awready)) break;
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) break;
    end
    m_vif.cb_slave.awvalid <= 0;
endtask

/* Task: write_data
 * Transmit AXI4-MM write transaction (W channel).
 * Input: wdata_t object which contains W transaction data.
 */
task axi4_mm_driver_slave::write_data(wdata_t data);
    bit is_running          = 1;
    bit data_avaiable       = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    data.assign_channel(wid, wdata, wstrb, wlast);
    assign_signals();
    m_vif.cb_slave.wvalid   <= 1;
    if (!data.get_last()) begin
        while (is_running && (1'b1 === m_vif.areset_n)) begin
            aclk_posedge();
            if (data_avaiable == 0) is_running = 0;
            else begin
                if ((1'b1 === m_vif.cb_slave.wvalid) && (1'b1 === m_vif.cb_slave.wready)) begin
                    data.assign_channel(wid, wdata, wstrb, wlast);
                    assign_signals();
                    if(data.get_last()) data_avaiable = 0;
                end
                else if ( timeout > 0) --timeout;
                else if (timeout == 0) is_running = 0;
            end
        end
    end
    else begin
        aclk_posedge();
        is_running      = 0;
        data_avaiable   = 0;
    end
    m_vif.cb_slave.wvalid   <= 0;
endtask

/* Task: write_response
 * Receives AXI4-MM write response transaction (B channel).
 * Input: reference to bresp_t object hich will be used to store B transaction data.
 */
task axi4_mm_driver_slave::write_response(ref bresp_t response);
    bit is_running          = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_slave.bready   <= 1;
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.bvalid) && (1'b1 === m_vif.cb_slave.bready)) begin
            is_running  = 0;
            response    = new(m_vif.cb_slave.bid, response_t'(m_vif.cb_slave.bresp));
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;
    end
    m_vif.cb_slave.bready <= 0;
endtask

/* Task: write
 * AXI4-MM write transaction. Sequentially transmits AW and W data, then receives B data.
 * Input:   request     - request_t object which contains AW channel transaction data.
 *          data        - wdata_t object which contains W channel transaction data.
 *          response    - bresp_t object which will be used to store B transaction data.
 */
task axi4_mm_driver_slave::write  (request_t request, wdata_t data, ref bresp_t response);  
    write_address   (request);
    write_data      (data);
    write_response  (response);
endtask

/* Task: write_multiple_address
 * Transmit AXI4-MM write address burst transaction.
 * Input: queue of wdata_t objects whose contain AW channel transaction data.
 */
task axi4_mm_driver_slave::write_multiple_address  (const ref request_array_t request);
    bit is_running          = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    int index = 0;
    if (request.size() > 0) begin
        request[index++].assign_channel(awid, awaddr, awlen, awsize, awburst);    
        assign_signals();
        m_vif.cb_slave.awvalid  <= 1'b1; 
        while (index != request.size() && (1'b1 === m_vif.areset_n)) begin
            aclk_posedge();
            if ((1'b1 === m_vif.cb_slave.awvalid) && (1'b1 === m_vif.cb_slave.awready)) begin
                request[index++].assign_channel(awid, awaddr, awlen, awsize, awburst);    
                assign_signals();                
            end
            else if ( timeout > 0) --timeout;
            else if (timeout == 0) is_running = 0;
        end
        aclk_posedge();
        m_vif.cb_slave.awvalid <= 0;
    end
endtask

/* Task: write_multiple_data
 * Transmit AXI4-MM write data burst transaction.
 * Input: queue of wdata_t objects whose contain W channel transaction data.
 */
task axi4_mm_driver_slave::write_multiple_data     (const ref wdata_array_t data);   
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    int index               = 0;
    while(index < data.size() && (1'b1 === m_vif.areset_n)) begin
        data[index].assign_channel(wid, wdata, wstrb, wlast);
        assign_signals();
        m_vif.cb_slave.wvalid   <= 1;
        while (1'b1 === m_vif.areset_n) begin
            aclk_posedge();
            if ((1'b1 === m_vif.cb_slave.wvalid) && (1'b1 === m_vif.cb_slave.wready)) begin
                data[index].assign_channel(wid, wdata, wstrb, wlast);
                assign_signals();
                if(data[index].get_last()) break;
            end
            else if ( timeout > 0) --timeout;
            else if (timeout == 0) break;
        end
        aclk_posedge();
        index++;
    end
    m_vif.cb_slave.wvalid <= 0;
endtask

/* Task: write_multiple_response
 * Receive AXI4-MM write response burst transaction (B channel).
 * Input: reference to queue of bresp_t objects whose will be used to store B transaction data.
 */  
task axi4_mm_driver_slave::write_multiple_response (ref bresp_t response[]);    
    bresp_t _response[$];
    bresp_t current_response;    
    bit is_running          = 0;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_slave.bready   <= 1;
    while (1'b1 === m_vif.areset_n) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.bvalid) && (1'b1 === m_vif.cb_slave.bready)) begin
            is_running = 1;
            current_response = new(m_vif.cb_slave.bid, response_t'(m_vif.cb_slave.bresp));
            _response.push_back(current_response);
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) break;
        if (is_running == 1 && (1'b0 === m_vif.cb_slave.bvalid)) begin
            is_running = 0;
            break;
        end
    end
    m_vif.cb_slave.bready   <= 0;
    response                = _response;
endtask

/* Task: write_multiple
 * AXI4-MM write burst transaction. Sequentially receives AW and W data, then sends B data.
 * Input:   request     - queue of wdata_t objects whose contain AW channel transaction data.
 *          data        - queue of wdata_t objects whose contain W channel transaction data.
 *          response    - reference to queue of bresp_t objects whose will be used to store B transaction data.
 */
task axi4_mm_driver_slave::write_multiple   (const ref request_array_t request, const ref wdata_array_t data, ref bresp_t response[]);  
    write_multiple_address   (request);
    write_multiple_data      (data);
    write_multiple_response  (response);
endtask

/* Task: write_ready
 * Sets write idle state (BREADY)
 * Input: Idle on (value = 0) or off (value = 1)
 */
task axi4_mm_driver_slave::write_ready (bit value = 1);
    m_vif.cb_slave.bready <= value;
endtask

/* Task: read_address
 * Transmits AXI4-MM read address transaction (AR channel).
 * Input: request_t object which contains AR channel transaction data.
 */ 
task axi4_mm_driver_slave::read_address (request_t request);
    bit is_running          = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    request.assign_channel(arid, araddr, arlen, arsize, arburst);    
    assign_signals();
    m_vif.cb_slave.arvalid  <= 1'b1; 
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.arvalid) && (1'b1 === m_vif.cb_slave.arready)) is_running = 0;
        else if (timeout > 0)   --timeout;
        else if (timeout == 0)  is_running = 0;
    end
    m_vif.cb_slave.arvalid <= 0;
endtask
            
/* Task: read_data
 * Transmits AXI4-MM read transaction (R channel).
 * Input: reference to rdata_t object which will be used to store R transaction data.
 */
task axi4_mm_driver_slave::read_data (ref rdata_t data);
    rdata_t _data           = new();
    bit is_running          = 1;
    bit data_avaiable       = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_slave.rready   <= 1;
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if (!data_avaiable) is_running = 0;
        if ((1'b1 === m_vif.cb_slave.rvalid) && (1'b1 === m_vif.cb_slave.rready)) begin
            _data.set_id(m_vif.cb_slave.rid);
            _data.set_response(response_t'(m_vif.cb_slave.rresp));
            _data.write(m_vif.cb_slave.rdata);    
            if (1'b1 === m_vif.cb_slave.rlast) data_avaiable = 0;            
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;
    end
    m_vif.cb_slave.rready   <= 0;
    data = _data;
endtask

/* Task: read
 * AXI4-MM read transaction. Sequentially transmits AR data, then receives R data.
 * Input:   request     - request_t object which contains AR transaction data..
 *          data        - reference to rdata_t object which will be used to store R transaction data.
 */
task axi4_mm_driver_slave::read (request_t request, ref rdata_t data);
    read_address(request);
    read_data   (data);
endtask

/* Task: read_multiple_address
 * Transmits AXI4-MM read address burst transaction.
 * Input: queue of request_t objects whose contain AR transaction data.
 */
task axi4_mm_driver_slave::read_multiple_address(const ref request_array_t request);
    bit is_running          = 1;
    int timeout             = (m_timeout > 0) ? m_timeout : -1;
    int index               = 0;
    m_vif.cb_slave.arvalid  <= 1'b1; 
    request[index++].assign_channel(arid, araddr, arlen, arsize, arburst);    
    assign_signals();
    while (is_running && (1'b1 === m_vif.areset_n)) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.arvalid) && (1'b1 === m_vif.cb_slave.arready)) begin
            if (index == request.size()) is_running = 0;
            else begin
                request[index++].assign_channel(arid, araddr, arlen, arsize, arburst);    
                assign_signals();                
            end 
        end
        else if ( timeout > 0) --timeout;
        else if (timeout == 0) is_running = 0;
    end
    m_vif.cb_slave.arvalid <= 0;
endtask

/* Task: read_multiple_data
 * Transmit AXI4-MM read data burst transaction.
 * Input: reference to queue of rdata_t objects whose will be used to store R transaction data.
 */
task axi4_mm_driver_slave::read_multiple_data (ref rdata_t data[]);    
    rdata_t _data[$];
    rdata_t current_data;  
    bit     is_running      = 0;
    bit     data_avaiable   = 0;
    int     timeout         = (m_timeout > 0) ? m_timeout : -1;
    m_vif.cb_slave.rready   <= 1;
    while (1'b1 === m_vif.areset_n) begin
        aclk_posedge();
        if ((1'b1 === m_vif.cb_slave.rvalid) && (1'b1 === m_vif.cb_slave.rready)) begin
            is_running = 1;
            if (data_avaiable == 0) begin
                current_data = new(m_vif.cb_slave.rid);
                current_data.write(m_vif.cb_slave.rdata);
                data_avaiable = 1;
            end
            else begin 
                current_data.write(m_vif.cb_slave.rdata);
            end
            if (1'b1 === m_vif.cb_slave.rlast) begin
                _data.push_back(current_data);
                data_avaiable = 0;
            end 
        end
        else if ( timeout > 0)  --timeout;
        else if (timeout == 0)  break;
        if (is_running == 1 && 1'b0 === m_vif.cb_slave.rvalid) break;
    end
    m_vif.cb_slave.rready   <= 0;
    data                    = _data; 
endtask

/* Task: read_multiple
 * AXI4-MM read transaction. Sequentially receives AR data, then sends R data.
 * Input:   request     - queue of request_t objects whose contain AR transaction data.
 *          data        - reference to queue of rdata_t objects whose will be used to store R transaction data.
 */
task axi4_mm_driver_slave::read_multiple     (const ref request_array_t request, ref rdata_t data[]);
    read_multiple_address   (request);
    read_multiple_data      (data);
endtask

/* Task: read_ready
 * Sets read idle state (RREADY)
 * Input: Idle on (value = 0) or off (value = 1)
 */
task axi4_mm_driver_slave::read_ready(bit value = 1);
    m_vif.cb_slave.rready <= value;
endtask

/* Task: aclk_posedge
 * Wait for rising edge of clock. (Clock period delay).
 * Input: count - number of clock cycles to wait.
 */
task axi4_mm_driver_slave::aclk_posedge(int count = 1);
    while (count--) begin
        @(m_vif.cb_slave);
    end
endtask
`endif /* AXI4_LITE_DRIVER_SLAVE_SVH */
