/* Interface: axi4_mm_if
 *
 * AXI4-MM interface.
 *
 * Parameters:
 *  DATA_BYTES      - Number of bytes in bus.
 *  ADDRESS_WIDTH   - Width of address signals.
 *  ID_WIDTH        - Width of ID signals.
 *  AWUSER_WIDTH    - Width of write address user signal.
 *  WUSER_WIDTH     - Width of write data user signal.
 *  BUSER_WIDTH     - Width of write response user signal.
 *  ARUSER_WIDTH    - Width of read address user signal.
 *  RUSER_WIDTH     - Width of read data user signal.
 *
 * Ports:
 *  aclk            - Clock. Used only for internal checkers and assertions
 *  areset_n        - Asynchronous active-low reset. Used only for internal checkers
 *                  and assertions
 */
interface axi4_mm_if #(
    int DATA_BYTES      = 8,
    int ADDRESS_WIDTH   = 32,
    int ID_WIDTH        = 8,
    int AWUSER_WIDTH    = 1,
    int WUSER_WIDTH     = 1,
    int BUSER_WIDTH     = 1,
    int ARUSER_WIDTH    = 1,
    int RUSER_WIDTH     = 1
) (
    input                           aclk,
    input                           areset_n
);

`ifdef SYNTHESIS
    logic [     ID_WIDTH-1:0]       awid;
    logic [ADDRESS_WIDTH-1:0]       awaddr;
    logic [              7:0]       awlen;
    logic [              2:0]       awsize;
    logic [              1:0]       awburst;
    logic                           awlock;
    logic [              3:0]       awcache;
    logic [              2:0]       awprot;
    logic [              3:0]       awqos;
    logic [              3:0]       awregion;
    logic [ AWUSER_WIDTH-1:0]       awuser;
    logic                           awvalid;
    logic                           awready;
    logic [     ID_WIDTH-1:0]       wid;
    logic [   DATA_BYTES-1:0][7:0]  wdata;
    logic [   DATA_BYTES-1:0]       wstrb;
    logic                           wlast;
    logic [  WUSER_WIDTH-1:0]       wuser;
    logic                           wvalid;
    logic                           wready;
    logic [     ID_WIDTH-1:0]       bid;
    logic [              1:0]       bresp;
    logic [  BUSER_WIDTH-1:0]       buser;
    logic                           bvalid;
    logic                           bready;
    logic [     ID_WIDTH-1:0]       arid;
    logic [ADDRESS_WIDTH-1:0]       araddr;
    logic [              7:0]       arlen;
    logic [              2:0]       arsize;
    logic [              1:0]       arburst;
    logic                           arlock;
    logic [              3:0]       arcache;
    logic [              2:0]       arprot;
    logic [              3:0]       arqos;
    logic [              3:0]       arregion;
    logic [ ARUSER_WIDTH-1:0]       aruser;
    logic                           arvalid;
    logic                           arready;
    logic [     ID_WIDTH-1:0]       rid;
    logic [   DATA_BYTES-1:0][7:0]  rdata;
    logic [              1:0]       rresp;
    logic                           rlast;
    logic [  RUSER_WIDTH-1:0]       ruser;
    logic                           rvalid;
    logic                           rready;
`else
    logic [     ID_WIDTH-1:0]       awid;
    logic [ADDRESS_WIDTH-1:0]       awaddr;
    logic [              7:0]       awlen;
    logic [              2:0]       awsize;
    logic [              1:0]       awburst;
    logic                           awlock;
    logic [              3:0]       awcache;
    logic [              2:0]       awprot;
    logic [              3:0]       awqos;
    logic [              3:0]       awregion;
    logic [ AWUSER_WIDTH-1:0]       awuser;
    logic                           awvalid;
    logic                           awready;
    logic [     ID_WIDTH-1:0]       wid;
    logic [   DATA_BYTES-1:0][7:0]  wdata;
    logic [   DATA_BYTES-1:0]       wstrb;
    logic                           wlast;
    logic [  WUSER_WIDTH-1:0]       wuser;
    logic                           wvalid;
    logic                           wready;
    logic [     ID_WIDTH-1:0]       bid;
    logic [              1:0]       bresp;
    logic [  BUSER_WIDTH-1:0]       buser;
    logic                           bvalid;
    logic                           bready;
    logic [     ID_WIDTH-1:0]       arid;
    logic [ADDRESS_WIDTH-1:0]       araddr;
    logic [              7:0]       arlen;
    logic [              2:0]       arsize;
    logic [              1:0]       arburst;
    logic                           arlock;
    logic [              3:0]       arcache;
    logic [              2:0]       arprot;
    logic [              3:0]       arqos;
    logic [              3:0]       arregion;
    logic [ ARUSER_WIDTH-1:0]       aruser;
    logic                           arvalid;
    logic                           arready;
    logic [     ID_WIDTH-1:0]       rid;
    logic [   DATA_BYTES-1:0][7:0]  rdata;
    logic [              1:0]       rresp;
    logic                           rlast;
    logic [  RUSER_WIDTH-1:0]       ruser;
    logic                           rvalid;
    logic                           rready;
`endif

`ifndef MODPORT_DISABLED
    modport master (
        input  aclk,
        input  areset_n,
        output awid,
        output awaddr,
        output awlen,
        output awsize,
        output awburst,
        output awlock,
        output awcache,
        output awprot,
        output awqos,
        output awregion,
        output awuser,
        output awvalid,
        input  awready,
        output wid,
        output wdata,
        output wstrb,
        output wlast,
        output wuser,
        output wvalid,
        input  wready,
        input  bid,
        input  bresp,
        input  buser,
        input  bvalid,
        output bready,
        output arid,
        output araddr,
        output arlen,
        output arsize,
        output arburst,
        output arlock,
        output arcache,
        output arprot,
        output arqos,
        output arregion,
        output aruser,
        output arvalid,
        input  arready,
        input  rid,
        input  rdata,
        input  rresp,
        input  rlast,
        input  ruser,
        input  rvalid,
        output rready
    );

    modport slave (
        input  aclk,
        input  areset_n,
        input  awid,
        input  awaddr,
        input  awlen,
        input  awsize,
        input  awburst,
        input  awlock,
        input  awcache,
        input  awprot,
        input  awqos,
        input  awregion,
        input  awuser,
        input  awvalid,
        output awready,
        input  wid,
        input  wdata,
        input  wstrb,
        input  wlast,
        input  wuser,
        input  wvalid,
        output wready,
        output bid,
        output bresp,
        output buser,
        output bvalid,
        input  bready,
        input  arid,
        input  araddr,
        input  arlen,
        input  arsize,
        input  arburst,
        input  arlock,
        input  arcache,
        input  arprot,
        input  arqos,
        input  arregion,
        input  aruser,
        input  arvalid,
        output arready,
        output rid,
        output rdata,
        output rresp,
        output rlast,
        output ruser,
        output rvalid,
        input  rready
    );

    modport monitor (
        input  aclk,
        input  areset_n,
        input  awid,
        input  awaddr,
        input  awlen,
        input  awsize,
        input  awburst,
        input  awlock,
        input  awcache,
        input  awprot,
        input  awqos,
        input  awregion,
        input  awuser,
        input  awvalid,
        input  awready,
        input  wid,
        input  wdata,
        input  wstrb,
        input  wlast,
        input  wuser,
        input  wvalid,
        input  wready,
        input  bid,
        input  bresp,
        input  buser,
        input  bvalid,
        input  bready,
        input  arid,
        input  araddr,
        input  arlen,
        input  arsize,
        input  arburst,
        input  arlock,
        input  arcache,
        input  arprot,
        input  arqos,
        input  arregion,
        input  aruser,
        input  arvalid,
        input  arready,
        input  rid,
        input  rdata,
        input  rresp,
        input  rlast,
        input  ruser,
        input  rvalid,
        input  rready
    );
`endif

`ifndef SYNTHESIS
  clocking cb_master @(posedge aclk);
        input  awid;
        input  awaddr;
        input  awlen;
        input  awsize;
        input  awburst;
        input  awlock;
        input  awcache;
        input  awprot;
        input  awqos;
        input  awregion;
        input  awuser;
        input  awvalid;
        inout  awready;
        input  wid;
        input  wdata;
        input  wstrb;
        input  wlast;
        input  wuser;
        input  wvalid;
        inout  wready;
        inout  bid;
        inout  bresp;
        inout  buser;
        inout  bvalid;
        input  bready;
        input  arid;
        input  araddr;
        input  arlen;
        input  arsize;
        input  arburst;
        input  arlock;
        input  arcache;
        input  arprot;
        input  arqos;
        input  arregion;
        input  aruser;
        input  arvalid;
        inout  arready;
        inout  rid;
        inout  rdata;
        inout  rresp;
        inout  rlast;
        inout  ruser;
        inout  rvalid;
        input  rready;
    endclocking

    modport cb_master_modport (
        input       areset_n,
        clocking    cb_master
    );

    clocking cb_slave @(posedge aclk);
        inout  awid;
        inout  awaddr;
        inout  awlen;
        inout  awsize;
        inout  awburst;
        inout  awlock;
        inout  awcache;
        inout  awprot;
        inout  awqos;
        inout  awregion;
        inout  awuser;
        inout  awvalid;
        input  awready;
        inout  wid;
        inout  wdata;
        inout  wstrb;
        inout  wlast;
        inout  wuser;
        inout  wvalid;
        inout  wready;
        input  bid;
        input  bresp;
        input  buser;
        input  bvalid;
        inout  bready;
        inout  arid;
        inout  araddr;
        inout  arlen;
        inout  arsize;
        inout  arburst;
        inout  arlock;
        inout  arcache;
        inout  arprot;
        inout  arqos;
        inout  arregion;
        inout  aruser;
        inout  arvalid;
        input  arready;
        input  rid;
        input  rdata;
        input  rresp;
        input  rlast;
        input  ruser;
        input  rvalid;
        inout  rready;
    endclocking

    modport cb_slave_modport (
        input       areset_n,
        clocking    cb_slave
    );
`endif
endinterface