/* Package: axi4_mm_pkg
 *
 * Logic AXI4-MM package.
 */
package axi4_mm_pkg;
    /* Enum: size_t
     * Transaction data size in bits (width). 
     */
    typedef enum logic [2:0] {
        SIZE_8                              = 3'b000,
        SIZE_16                             = 3'b001,
        SIZE_32                             = 3'b010,
        SIZE_64                             = 3'b011,
        SIZE_128                            = 3'b100,
        SIZE_256                            = 3'b101,
        SIZE_512                            = 3'b110,
        SIZE_1024                           = 3'b111
    } size_t;

    /* Function : default_size
     * Returns size_t enum value according to AXI data width (bytes input).
     * Input    : bytes - axi data size in bytes.
     * Output   : size_t enum value.
     */
    function size_t default_size(int bytes);
        if      (bytes >= 128)  default_size = SIZE_1024;
        else if (bytes >=  64)  default_size = SIZE_512;
        else if (bytes >=  32)  default_size = SIZE_256;
        else if (bytes >=  16)  default_size = SIZE_128;
        else if (bytes >=   8)  default_size = SIZE_64;
        else if (bytes >=   4)  default_size = SIZE_32;
        else if (bytes >=   2)  default_size = SIZE_16;
        else                    default_size = SIZE_8;
    endfunction

    /* Enum : burst_t 
     * Burst type.
     * BURST_TYPE_FIXED                     - Fixed burst. Address is not incremented.
     * BURST_TYPE_INCR                      - Incremented burst. Increment is based on data size (see size_t).
     * BURST_TYPE_WRAP                      - Wrapped burst. Similiar to increment, but address wrap to base address if upeer address is reached.
     * BURST_TYPE_RESERVED                  - Reserved. No use.
     */
    typedef enum logic [1:0] {
        BURST_TYPE_FIXED                    = 2'b00,
        BURST_TYPE_INCR                     = 2'b01,
        BURST_TYPE_WRAP                     = 2'b10,
        BURST_TYPE_RESERVED                 = 2'b11
    } burst_t;

    /* Enum : lock_t
     * Lock type.
     * LOCK_NORMAL_ACCESS                   - Normal access.
     * LOCK_EXCLUSIVE_ACCESS                - Exclusive access.
     */
    typedef enum logic [0:0] {
        LOCK_NORMAL_ACCESS                  = 1'b0,
        LOCK_EXCLUSIVE_ACCESS               = 1'b1
    } lock_t;

    /* Enum : bufferable_transaction_t
     * Bufferable transaction.
     * BUFFERABLE_TRANSACTION_UNBUFFERABLE  - Transaction can't be buffered.
     * BUFFERABLE_TRANSACTION_BUFFERABLE    - Transaction can be buffered.
     */
    typedef enum logic [0:0] {
        BUFFERABLE_TRANSACTION_UNBUFFERABLE = 1'b0,
        BUFFERABLE_TRANSACTION_BUFFERABLE   = 1'b1
    } bufferable_transaction_t;

    /* Enum : modifiable_transaction_t
     * Modifiable transaction.
     * MODIFIABLE_TRANSACTION_UNMODIFIABLE  - Transaction can't be modified.
     * MODIFIABLE_TRANSACTION_MODIFIABLE    - Transaction can be modified.
     */
    typedef enum logic [0:0] {
        MODIFIABLE_TRANSACTION_UNMODIFIABLE = 1'b0,
        MODIFIABLE_TRANSACTION_MODIFIABLE   = 1'b1
    } modifiable_transaction_t;

    /* Enum: write_allocate_t
     * Write allocate.
     * WRITE_ALLOCATE_SET                   - Set write allocate bit.
     * WRITE_ALLOCATE_CLEAR                 - Clear write allocate bit.
     */
    typedef enum logic [0:0] {
        WRITE_ALLOCATE_SET                  = 1'b1,
        WRITE_ALLOCATE_CLEAR                = 1'b0
    } write_allocate_t;

    /* Enum: read_allocate_t
     * Read allocate.
     * READ_ALLOCATE_SET                    - Set read allocate bit.
     * READ_ALLOCATE_CLEAR                  - Clear read allocate bit.
     */
    typedef enum logic [0:0] {
        READ_ALLOCATE_SET                   = 1'b1,
        READ_ALLOCATE_CLEAR                 = 1'b0
    } read_allocate_t;

    /* Enum: cache_t
     * AxCACHE type. Specifies the attributes of transaction.
     * WRITE_ALLOCATE                       - Write transaction was previously allocated.
     * READ_ALLOCATE                        - Read transaction was previously allocated.
     * TRANSACITON_MODIFIABLE               - Transaction parameters can be modified.
     * TRANSACTION_BUFFERABLE               - Transaction can be delayed (applicable to writes)
     */
    typedef struct packed {
        write_allocate_t                    write_allocate;
        read_allocate_t                     read_allocate;
        modifiable_transaction_t            modifiable_transaction;
        bufferable_transaction_t            bufferable_transaction;
    } cache_t;       

    /* Const: DEFAULT_CACHE
     *
     * Default transaction attributes.
     */
    localparam cache_t DEFAULT_CACHE = '{
        write_allocate                      : WRITE_ALLOCATE_CLEAR,
        read_allocate                       : READ_ALLOCATE_CLEAR,
        modifiable_transaction              : MODIFIABLE_TRANSACTION_UNMODIFIABLE,
        bufferable_transaction              : BUFFERABLE_TRANSACTION_BUFFERABLE
    };

    /* Enum: privilege_access_t
     *
     * Privilege access.
     *
     * PRIVILEGE_ACCESS_UNPRIVILEGED        - Unprivileged access.
     * PRIVILEGE_ACCESS_PRIVILEGED          - Privileged access.
     */
    typedef enum logic [0:0] {
        PRIVILEGE_ACCESS_UNPRIVILEGED       = 1'b0,
        PRIVILEGE_ACCESS_PRIVILEGED         = 1'b1
    } privilege_access_t;

    /* Enum: security_access_t
     *
     * Security access.
     *
     * SECURITY_ACCESS_SECURE               - Secure access.
     * SECURITY_ACCESS_NON_SECURE           - Non-secure access.
     */
    typedef enum logic [0:0] {
        SECURITY_ACCESS_SECURE              = 1'b0,
        SECURITY_ACCESS_NON_SECURE          = 1'b1
    } security_access_t;

    /* Enum: type_access_t
     *
     * Type access.
     *
     * TYPE_ACCESS_DATA                     - Data access.
     * TYPE_ACCESS_INSTRUCTION              - Instruction access.
     */
    typedef enum logic [0:0] {
        TYPE_ACCESS_DATA                    = 1'b0,
        TYPE_ACCESS_INSTRUCTION             = 1'b1
    } type_access_t;

    /* Enum: access_t
     *
     * Access.
     *
     * type_access                          - Type access.
     * security_access                      - Security access.
     * privilege_access                     - Privilege access.
     */
    typedef struct packed {
        type_access_t                       type_access;
        security_access_t                   security_access;
        privilege_access_t                  privilege_access;
    } access_t;

    /* Const: DEFAULT_DATA_ACCESS
     *
     * Default data access permission.
     */
    localparam access_t DEFAULT_DATA_ACCESS = '{
        type_access                         : TYPE_ACCESS_DATA,
        security_access                     : SECURITY_ACCESS_SECURE,
        privilege_access                    : PRIVILEGE_ACCESS_UNPRIVILEGED
    };

    /* Const: DEFAULT_INSTRUCTION_ACCESS
     *
     * Default data access permission.
     */
    localparam access_t DEFAULT_INSTRUCTION_ACCESS = '{
        type_access                         : TYPE_ACCESS_INSTRUCTION,
        security_access                     : SECURITY_ACCESS_SECURE,
        privilege_access                    : PRIVILEGE_ACCESS_UNPRIVILEGED
    };

    /* Enum: response_t
     *
     * Write/read response.
     *
     * RESPONSE_OKAY                        - Normal access success.
     * RESPONSE_EXOKAY                      - Exclusive access okay.
     * RESPONSE_SLVERR                      - Slave error.
     * RESPONSE_DECERR                      - Decode error.
     */
    typedef enum logic [1:0] {
        RESPONSE_OKAY                       = 2'b00,
        RESPONSE_EXOKAY                     = 2'b01,
        RESPONSE_SLVERR                     = 2'b10,
        RESPONSE_DECERR                     = 2'b11
    } response_t;
    
    /* Class: request_t
     * Object contains data and functions related to AW/AR channel transactions.
     */
    class request_t #(
        int ID_WIDTH        = 8,
        int ADDRESS_WIDTH   = 32,
        int BYTES           = 8
    );
        /* Packed data types with parametrized widths */
        typedef bit   [     ID_WIDTH-1:0]   id_type;
        typedef bit   [ADDRESS_WIDTH-1:0]   address_type;
        typedef bit   [              7:0]   length_type;
        typedef size_t                      size_type;
        typedef burst_t                     burst_type;

        /* Logic data types with parametrized widths for driving AW/AR channel */
        typedef logic [ID_WIDTH-1:0]        id_logic_type;
        typedef logic [ADDRESS_WIDTH-1:0]   address_logic_type;
        typedef logic [7:0]                 length_logic_type;
        typedef logic [2:0]                 size_logic_type;
        typedef logic [1:0]                 burst_logic_type;
        
        /* Local variables containing data about transaction */
        local id_type                       id;
        local address_type                  address;
        local length_type                   length;
        local size_type                     size;
        local burst_type                    burst;

        /* Function: new
         * Constructor. Initializes object local variables.
         * Inputs: id, address, length, size, burst - AW/AR transactions parameters.
         */
        function new(id_type id = '0, address_type address = '0, length_type length = '0, size_type size = default_size(BYTES), burst_type burst = BURST_TYPE_INCR);
            this.id         = id;
            this.address    = address;
            this.length     = length;
            this.size       = size;
            this.burst      = burst;
        endfunction

        /* Function: set_id
         * Set object local variable : id.
         */
        function void set_id(id_type id = '0);
            this.id         = id;
        endfunction

        /* Function: set_address
         * Set object local variable : address.
         */
        function void set_address(address_type address = '0);
            this.address    = address;
        endfunction

        /* Function: set_length
         * Set object local variable : length.
         */
        function void set_length(length_type length = '0);
            this.length     = length;
        endfunction

        /* Function: set_size
         * Set object local variable : size.
         */
        function void set_size(size_type size = default_size(BYTES));
            this.size       = size;
        endfunction
    
        /* Function: set_burst
         * Set object local variable : burst.
         */
        function void set_burst(burst_type burst = BURST_TYPE_FIXED);
            this.burst      = burst;
        endfunction

        /* Function: get_id
         * Returns object local variable : id.
         */
        function id_type get_id;
            get_id          = this.id;
        endfunction

        /* Function: get_address
         * Returns object local variable : address.
         */
        function address_type get_address;
            get_address     = this.address;
        endfunction

        /* Function: get_length
         * Returns object local variable : length.
         */
        function length_type get_length;
            get_length      = this.length;
        endfunction

        /* Function: get_size
         * Returns object local variable : size.
         */
        function size_type get_size;
            get_size        = this.size;
        endfunction

        /* Function: get_burst
         * Returns object local variable : burst.
         */    
        function burst_type get_burst;
            get_burst       = this.burst;
        endfunction
        
        /* Function: show
         * Displays AW/AR transaction data in simulator.
         */
        function void show;
            $display("AXI transaction request:");
            $display("ID = %0d, ADDRESS = %h, LENGTH = %0d, SIZE = %s, BURST = %s", this.id, this.address, this.length+1, this.size.name(), this.burst.name());
        endfunction
        
        /* Function: assign_channel
         * Assigns data to AW/AR channel.
         * Input: references to channel signals.
         */ 
        function void assign_channel(ref id_logic_type AxID, ref address_logic_type AxADDR, ref length_logic_type AxLEN, ref size_logic_type AxSIZE, ref burst_logic_type AxBURST);
            AxID    = id_logic_type'        (this.id);
            AxADDR  = address_logic_type'   (this.address);
            AxLEN   = length_logic_type'    (this.length);
            AxSIZE  = size_logic_type'      (this.size);
            AxBURST = burst_logic_type'     (this.burst);
        endfunction
    endclass     

    /* Class: wdata_t
     * Object contains write data information (W channel).
     */
    class wdata_t #(
        int ID_WIDTH    = 8,
        int BYTES       = 8
    );
        /* Packed data types with parametrized widths */
        typedef bit   [ID_WIDTH-1:0]        id_type;
        typedef bit   [   BYTES-1:0][7:0]   data_type;
        typedef bit   [   BYTES-1:0]        byte_enable_type;
        typedef struct {
            data_type           data;
            byte_enable_type    strobe;
        }                                   write_data_type;

        /* Logic data types with parametrized widths for driving W channel */
        typedef logic [ID_WIDTH-1:0]        id_logic_type;
        typedef logic [   BYTES-1:0][7:0]   data_logic_type;
        typedef logic [   BYTES-1:0]        byte_enable_logic_type;

        /* Local variables containing data about transaction */
        local id_type                       id;
        local write_data_type               data[$];
        local int                           read_index;

        /* Function: new
         * Constructor. Initializes object local variables.
         * Inputs: id - W transaction parameter.
         */
        function new(id_type id = '0);
            this.id     = id;
            read_index  = -1;
        endfunction

        /* Function: set_id
         * Set object local variable : id.
         */
        function void set_id(id_type id = '0);
            this.id = id;
        endfunction

        /* Function: write
         * Write data to local data queue.
         */
        function void write(data_type data = '0, byte_enable_type strobe = '1); 
            write_data_type     wdata;
            wdata.data          = data;
            wdata.strobe        = strobe;                
            this.data.push_back (wdata);
        endfunction         

        /* Function: get_id
         * Returns object local variable : id.
         */
        function id_type get_id;
            get_id = this.id;
        endfunction

        /* Function: get_data
         * Returns data from local data queue with specified index.
         * Input:   index of data in queue.
         * Output:  structure with data and strobe (write_data_type)
         * Default: returns last data from queue.
         */
        function write_data_type get_data(int index = this.data.size()-1);
            get_data = this.data[index];
        endfunction

        /* Function: read
         * Returns data from local data queue with index based on internal read pointer.
         * Output: structure with data and strobe (write_data_type).
         */
        function write_data_type read;
            read_index++;
            read = this.data[read_index];
        endfunction

        /* Function: get_last
         * Return 1 if next read will return last data in queue, else 0.
         */
        function bit get_last;
            if (read_index == (this.data.size() - 1))   get_last = 1;
            else                                        get_last = 0;
        endfunction

        /* Function: set_read_index
         * Sets internal read pointer. Can be used to send same data one more time or skip data.
         */
        function void set_read_index(int index);
            this.read_index = index-1;
        endfunction

        /* Function: size
         * Returns data queue size.
         */
        function int size;
            size = this.data.size();
        endfunction

        /* Function: delete
         * Deletes data with indexes in range <lindex, rindex>.
         */
        function void delete(int lindex = 0, int rindex = lindex);
            for(int i = rindex; i >= lindex; i--) begin
                this.data.delete(i);
            end
        endfunction

        /* Function: show
         * Displays W transaction data and read pointer value in simulator.
         */
        function void show;
            $display("ID = %h", id);
            $display("Read index = %0d", read_index);
            for(int i = 0; i < data.size(); i++) begin
                $display("%0d -> data = %h, strobe = %h", i, data[i].data, data[i].strobe);
            end            
        endfunction 

        /* Function: assign_channel
         * Assigns data to W channel.
         * Input: references to channel signals.
         */ 
        function void assign_channel(ref id_logic_type WID, ref data_logic_type WDATA, ref byte_enable_logic_type WSTRB, ref logic WLAST);
            write_data_type data    = read();
            bit             last    = get_last();
            WID                     = id_logic_type'            (id);
            WDATA                   = data_logic_type'          (data.data);
            WSTRB                   = byte_enable_logic_type'   (data.strobe);
            WLAST                   = logic'                    (last);
        endfunction
    endclass

    /* Class: bresp_t
     * Object contains write response information (B channel).
     */
    class bresp_t #(
        int ID_WIDTH = 8
    );
        /* Packed data types with parametrized widths */
        typedef bit   [ID_WIDTH-1:0]    id_type;

        /* Logic data types with parametrized widths for driving B channel */
        typedef logic [ID_WIDTH-1:0]    id_logic_type;
        typedef logic [1:0]             response_logic_type;
        
        /* Local variables containing data about transaction */
        local id_type                   id;
        local response_t                response;

        /* Function: new
         * Constructor. Initializes object local variables.
         * Inputs: id, resp - B transaction parameters.
         */
        function new(id_type id = '0, response_t resp = RESPONSE_OKAY);
            this.id         = id;
            this.response   = resp;
        endfunction

        /* Function: set_id
         * Set object local variable : id.
         */
        function void set_id(id_type id = '0);
            this.id = id;
        endfunction

        /* Function: set_response
         * Set object local variable : response.
         */
        function void set_response(response_t resp = RESPONSE_OKAY);
            this.response = resp;
        endfunction

        /* Function: get_id
         * Returns object local variable : id.
         */
        function id_type get_id;
            get_id = this.id;
        endfunction

        /* Function: get_response
         * Returns object local variable : response.
         */
        function response_t get_response;
            get_response = this.response;
        endfunction
        
        /* Function: show
         * Displays B transaction data and read pointer value in simulator.
         */
        function void show;
            $display("ID = %h, RESP = %s", id, response.name());          
        endfunction 

        /* Function: assign_channel
         * Assigns data to B channel.
         * Input: references to channel signals.
         */ 
        function void assign_channel(ref id_logic_type BID, ref response_logic_type BRESP);
            BID     = id_logic_type'        (get_id());
            BRESP   = response_logic_type'  (get_response());
        endfunction        
    endclass
    
    /* Class: rdata_t
     * Object contains read data information (R channel).
     */
    class rdata_t #(
        int ID_WIDTH    = 8,
        int BYTES       = 8
    );
        /* Packed data types with parametrized widths */
        typedef bit     [ID_WIDTH-1:0]          id_type;
        typedef bit     [   BYTES-1:0][7:0]     data_type;

        /* Logic data types with parametrized widths for driving R channel */        
        typedef logic   [ID_WIDTH-1:0]          id_logic_type;
        typedef logic   [   BYTES-1:0][7:0]     data_logic_type;
        typedef logic   [1:0]                   response_logic_type;

        /* Local variables containing data about transaction */
        local id_type                           id;
        local data_type                         data[$];
        local response_t                        response;
        local int                               read_index;

        /* Function: new
         * Constructor. Initializes object local variables.
         * Inputs: id, resp - R transaction parameters.
         */
        function new(id_type id = '0, response_t resp = RESPONSE_OKAY);
            this.id     = id;
            this.response = resp;
            read_index  = -1;
        endfunction

        /* Function: set_id
         * Set object local variable : id.
         */
        function void set_id(id_type id = '0);
            this.id = id;
        endfunction

        /* Function: set_response
         * Set object local variable : id.
         */
        function void set_response(response_t resp = RESPONSE_OKAY);
            this.response = resp;
        endfunction

        /* Function: write 
         * Write data to local data queue.
         */
        function void write(data_type data = '0);               
            this.data.push_back(data);
        endfunction      
    
        /* Function: read
         * Returns data from local data queue with index based on internal read pointer.
         * Output: structure with data and strobe (write_data_type).
         */        
        function data_type read;
            read_index++;
            read = this.data[read_index];
        endfunction   

        /* Function: get_id
         * Returns object local variable : id.
         */
        function id_type get_id;
            get_id = this.id;
        endfunction

        /* Function: get_response
         * Returns object local variable : response.
         */
        function response_t get_response;
            get_response = this.response;
        endfunction

        /* Function: get_data
         * Returns data from local data queue with specified index.
         * Input:   index of data in queue.
         * Default: returns last data from queue.
         */
        function data_type get_data(int index = this.data.size()-1);
            get_data = this.data[index];
        endfunction

        /* Function: get_last
         * Return 1 if next read will return last data in queue, else 0.
         */
        function bit get_last;
            if (read_index == (this.data.size() - 1))   get_last = 1;
            else                                        get_last = 0;
        endfunction

        /* Function: set_read_index
         * Sets internal read pointer. Can be used to send same data one more time or skip data.
         */
        function void set_read_index(int index);
            this.read_index = index-1;
        endfunction

        /* Function: size
         * Returns data queue size.
         */
        function int size;
            size = this.data.size();
        endfunction

        /* Function: delete
         * Deletes data with indexes in range <lindex, rindex>.
         */
        function void delete(int lindex = 0, int rindex = lindex);
            for(int i = rindex; i >= lindex; i--) begin
                this.data.delete(i);
            end
        endfunction

        /* Function: show
         * Displays R transaction data and read pointer value in simulator.
         */
        function void show;
            $display("ID = %h", id);
            $display("Read index = %0d", read_index);
            for(int i = 0; i < data.size(); i++) begin
                $display("%0d -> data = %h", i, data[i]);
            end            
        endfunction 

        /* Function: assign_channel
         * Assigns data to R channel.
         * Input: references to channel signals.
         */
        function void assign_channel(ref id_logic_type RID, ref data_logic_type RDATA, ref logic RLAST, ref response_logic_type RRESP);
            RID     = id_logic_type'(get_id());
            RDATA   = data_logic_type'(read());
            RLAST   = logic'(get_last());
            RRESP   = response_logic_type'(get_response());
        endfunction
    endclass
endpackage
