`timescale 1 ns / 1 ns

`include "svunit_defines.svh"

module axi_ram_unit_test;
    import svunit_pkg::svunit_testcase;
    import unit_test_pkg::*;
    import axi4_mm_pkg::*;

    string name = "axi_ram_unit_test";
    svunit_testcase svunit_ut;

    localparam real TIME_BASE = 1000.0;
    localparam real CLOCK_50 = (TIME_BASE / 50.00);

    logic reset_n_50mhz = 1'b0;
    logic clk_50mhz = 0;

    initial forever #(CLOCK_50/2) clk_50mhz = ~clk_50mhz;

    parameter verbose = 1;

    // ----------------------------------
    // ------ Interfaces & Drivers ------
    // ----------------------------------
    parameter DATA_BYTES    =  4;
    parameter ADDRESS_WIDTH = 21;
    parameter ID_WIDTH      = 12;

    /* Parametrized data types from axi4_mm package */
    typedef axi4_mm_pkg::request_t    # (.ID_WIDTH(ID_WIDTH), .ADDRESS_WIDTH(ADDRESS_WIDTH), .BYTES(DATA_BYTES) ) request_t;
    typedef request_t                                                                                             request_array_t[];
    typedef axi4_mm_pkg::wdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                wdata_t;
    typedef wdata_t                                                                                               wdata_array_t[];
    typedef axi4_mm_pkg::bresp_t      # (.ID_WIDTH(ID_WIDTH) )                                                    bresp_t;
    typedef axi4_mm_pkg::rdata_t      # (.ID_WIDTH(ID_WIDTH), .BYTES(DATA_BYTES) )                                rdata_t;
    typedef axi4_mm_pkg::response_t                                                                               response_t;

    axi4_mm_if #(
        .DATA_BYTES     (DATA_BYTES),
        .ADDRESS_WIDTH  (ADDRESS_WIDTH),
        .ID_WIDTH       (ID_WIDTH)
    ) axi4_mm_slave (
        .aclk           (clk_50mhz),
        .areset_n       (reset_n_50mhz)
    );

    axi4_mm_driver_slave #(
        .DATA_BYTES     (DATA_BYTES),
        .ID_WIDTH       (ID_WIDTH),
        .ADDRESS_WIDTH  (ADDRESS_WIDTH)
    ) axi4_mm_slave_drv = new (axi4_mm_slave);

    // ----------------------------------
    // ------- DUT initialization -------
    // ----------------------------------
    axi_ram_wrapper #(
        .ADDR_WIDTH     (ADDRESS_WIDTH),
        .ID_WIDTH       (ID_WIDTH)
    ) dut (
        .clk            (clk_50mhz),
        .reset_n        (reset_n_50mhz),

        .axi4_mm_slave  (axi4_mm_slave) 
    );

    function void build();
        svunit_ut = new(name);
    endfunction

    task setup();
        svunit_ut.setup();
    
        #120;
        @(posedge clk_50mhz);
        reset_n_50mhz <= 1'b1;

        axi4_mm_slave_drv.aclk_posedge();
        axi4_mm_slave_drv.reset();
        axi4_mm_slave_drv.aclk_posedge();
    endtask

    task teardown();
        svunit_ut.teardown();
        reset_n_50mhz = 0;
        axi4_mm_slave_drv.reset();
    endtask

`SVUNIT_TESTS_BEGIN
  `SVTEST(basic)
    request_t request    = new();
    wdata_t   write_data = new();
    wdata_t::data_type _write_data;
    bresp_t   write_response;
    rdata_t   read_data;  
    
    wdata_t::write_data_type write_data_element;
    rdata_t::data_type       read_data_element;

    for (int i=0; i<256; i++) begin
        foreach (_write_data[k]) begin
            _write_data[k] = $urandom;
        end

        write_data.write(_write_data, '1);
    end

    request.set_address(21'h0);
    request.set_length(255);
    request.set_burst(BURST_TYPE_INCR);
    request.set_id(12'h1);
    if (verbose) begin
        request.show();
    end
    
    axi4_mm_slave_drv.write(request, write_data, write_response);

    if (verbose) begin
        write_response.show();
    end
    
    axi4_mm_slave_drv.read(request, read_data);

    if ((write_data.size() !== read_data.size()) || verbose) begin
        $display("write_data.size() = %0d !== read_data.size() = %0d", write_data.size(), read_data.size());
    end
    `FAIL_UNLESS_EQUAL(write_data.size(), read_data.size())

    for (int i=0; i<read_data.size(); i++) begin
        write_data_element = write_data.get_data(i);
        read_data_element  = read_data.get_data(i);
        if ((write_data_element.data !== read_data_element) || verbose) begin
            $display("write_data_element[%0d] = %0d !== read_data_element[%0d] = %0d", i, write_data_element.data, i, read_data_element);
        end
        `FAIL_UNLESS_EQUAL(write_data_element.data, read_data_element)
    end

  `SVTEST_END
`SVUNIT_TESTS_END

endmodule