`include "modport.svh"

`timescale 1ps / 1ps
`default_nettype none

module axi_ram_wrapper #(
    // Width of address bus in bits
    parameter ADDR_WIDTH = 21,
    // Width of ID signal
    parameter ID_WIDTH = 12,
    // Extra pipeline register on output
    parameter PIPELINE_OUTPUT = 0
) (
    input  wire                   clk,
    input  wire                   reset_n,

    `MODPORT(axi4_mm_if, slave)   axi4_mm_slave 
);

// Width of data bus in bits
localparam DATA_WIDTH = 32;
// Width of wstrb (width of data bus in words)
localparam STRB_WIDTH = (DATA_WIDTH/8);

// Write address channel
wire [ID_WIDTH-1:0]    s_axi_awid;
wire [ADDR_WIDTH-1:0]  s_axi_awaddr;
wire [7:0]             s_axi_awlen;
wire [2:0]             s_axi_awsize;
wire [1:0]             s_axi_awburst;
wire                   s_axi_awlock;
wire [3:0]             s_axi_awcache;
wire [2:0]             s_axi_awprot;
wire                   s_axi_awvalid;
wire                   s_axi_awready;

assign s_axi_awid            = axi4_mm_slave.awid;
assign s_axi_awaddr          = axi4_mm_slave.awaddr;
assign s_axi_awlen           = axi4_mm_slave.awlen;
assign s_axi_awsize          = axi4_mm_slave.awsize;
assign s_axi_awburst         = axi4_mm_slave.awburst;
assign s_axi_awlock          = axi4_mm_slave.awlock;
assign s_axi_awcache         = axi4_mm_slave.awcache;
assign s_axi_awprot          = axi4_mm_slave.awprot;
assign s_axi_awvalid         = axi4_mm_slave.awvalid;
assign axi4_mm_slave.awready = s_axi_awready;

// Write data channel
wire [DATA_WIDTH-1:0]  s_axi_wdata;
wire [STRB_WIDTH-1:0]  s_axi_wstrb;
wire                   s_axi_wlast;
wire                   s_axi_wvalid;
wire                   s_axi_wready;

assign s_axi_wdata[ 7: 0]   = axi4_mm_slave.wdata[0];
assign s_axi_wdata[15: 8]   = axi4_mm_slave.wdata[1];
assign s_axi_wdata[23:16]   = axi4_mm_slave.wdata[2];
assign s_axi_wdata[31:24]   = axi4_mm_slave.wdata[3];
assign s_axi_wstrb          = axi4_mm_slave.wstrb;
assign s_axi_wlast          = axi4_mm_slave.wlast;
assign s_axi_wvalid         = axi4_mm_slave.wvalid;
assign axi4_mm_slave.wready = s_axi_wready;

// Write response channel
wire [ID_WIDTH-1:0]    s_axi_bid;
wire [1:0]             s_axi_bresp;
wire                   s_axi_bvalid;
wire                   s_axi_bready;

assign axi4_mm_slave.bid    = s_axi_bid;
assign axi4_mm_slave.bresp  = s_axi_bresp;
assign axi4_mm_slave.bvalid = s_axi_bvalid;
assign axi4_mm_slave.buser  = '0;
assign s_axi_bready = axi4_mm_slave.bready;

// Read address channel
wire [ID_WIDTH-1:0]    s_axi_arid;
wire [ADDR_WIDTH-1:0]  s_axi_araddr;
wire [7:0]             s_axi_arlen;
wire [2:0]             s_axi_arsize;
wire [1:0]             s_axi_arburst;
wire                   s_axi_arlock;
wire [3:0]             s_axi_arcache;
wire [2:0]             s_axi_arprot;
wire                   s_axi_arvalid;
wire                   s_axi_arready;

assign s_axi_arid            = axi4_mm_slave.arid;
assign s_axi_araddr          = axi4_mm_slave.araddr;
assign s_axi_arlen           = axi4_mm_slave.arlen;
assign s_axi_arsize          = axi4_mm_slave.arsize;
assign s_axi_arburst         = axi4_mm_slave.arburst;
assign s_axi_arlock          = axi4_mm_slave.arlock;
assign s_axi_arcache         = axi4_mm_slave.arcache;
assign s_axi_arprot          = axi4_mm_slave.arprot;
assign s_axi_arvalid         = axi4_mm_slave.arvalid;
assign axi4_mm_slave.arready = s_axi_arready;

// Read data channel
wire [ID_WIDTH-1:0]    s_axi_rid;
wire [DATA_WIDTH-1:0]  s_axi_rdata;
wire [1:0]             s_axi_rresp;
wire                   s_axi_rlast;
wire                   s_axi_rvalid;
wire                   s_axi_rready;

assign axi4_mm_slave.rid      = s_axi_rid;
assign axi4_mm_slave.rdata[0] = s_axi_rdata[ 7: 0];
assign axi4_mm_slave.rdata[1] = s_axi_rdata[15: 8];
assign axi4_mm_slave.rdata[2] = s_axi_rdata[23:16];
assign axi4_mm_slave.rdata[3] = s_axi_rdata[31:24];
assign axi4_mm_slave.rresp    = s_axi_rresp;
assign axi4_mm_slave.rlast    = s_axi_rlast;
assign axi4_mm_slave.ruser    = '0;
assign axi4_mm_slave.rvalid   = s_axi_rvalid;
assign s_axi_rready           = axi4_mm_slave.rready;

axi_ram #(
    .DATA_WIDTH         (DATA_WIDTH),
    .ADDR_WIDTH         (ADDR_WIDTH),
    .STRB_WIDTH         (STRB_WIDTH),
    .ID_WIDTH           (ID_WIDTH),
    .PIPELINE_OUTPUT    (PIPELINE_OUTPUT)
) u_axi_ram (
    .clk                (clk),
    .rst                (~reset_n),

    .s_axi_awid         (s_axi_awid),
    .s_axi_awaddr       (s_axi_awaddr),
    .s_axi_awlen        (s_axi_awlen),
    .s_axi_awsize       (s_axi_awsize),
    .s_axi_awburst      (s_axi_awburst),
    .s_axi_awlock       (s_axi_awlock),
    .s_axi_awcache      (s_axi_awcache),
    .s_axi_awprot       (s_axi_awprot),
    .s_axi_awvalid      (s_axi_awvalid),
    .s_axi_awready      (s_axi_awready),
    .s_axi_wdata        (s_axi_wdata),
    .s_axi_wstrb        (s_axi_wstrb),
    .s_axi_wlast        (s_axi_wlast),
    .s_axi_wvalid       (s_axi_wvalid),
    .s_axi_wready       (s_axi_wready),
    .s_axi_bid          (s_axi_bid),
    .s_axi_bresp        (s_axi_bresp),
    .s_axi_bvalid       (s_axi_bvalid),
    .s_axi_bready       (s_axi_bready),
    .s_axi_arid         (s_axi_arid),
    .s_axi_araddr       (s_axi_araddr),
    .s_axi_arlen        (s_axi_arlen),
    .s_axi_arsize       (s_axi_arsize),
    .s_axi_arburst      (s_axi_arburst),
    .s_axi_arlock       (s_axi_arlock),
    .s_axi_arcache      (s_axi_arcache),
    .s_axi_arprot       (s_axi_arprot),
    .s_axi_arvalid      (s_axi_arvalid),
    .s_axi_arready      (s_axi_arready),
    .s_axi_rid          (s_axi_rid),
    .s_axi_rdata        (s_axi_rdata),
    .s_axi_rresp        (s_axi_rresp),
    .s_axi_rlast        (s_axi_rlast),
    .s_axi_rvalid       (s_axi_rvalid),
    .s_axi_rready       (s_axi_rready)
);

endmodule